from argparse import ArgumentParser, RawDescriptionHelpFormatter
from random import choice, randrange, seed

import sys

argparser = ArgumentParser(description='''\
  Generates memory reference strings.
''', formatter_class=RawDescriptionHelpFormatter)

argparser.add_argument('-S', '--seed', type=int, default=123)
argparser.add_argument('-L', '--length', type=int, default=10)
argparser.add_argument('-T', '--threads', type=int, default=1)
argparser.add_argument('-M', '--memory', type=int, default=100)
argparser.add_argument('-C', '--cache', type=int, default=10)
argparser.add_argument('-H', '--hotzone', type=int, default=20) # for locality
argparser.add_argument('-A', '--algo', default='random',
  choices=['random', 'local'])

args = None

def random_gen(a, b):
  while True:
    yield randrange(a, b)

def local_gen():
  xs = list(1 + randrange(args.memory) for _ in range(args.hotzone))
  while True:
    i = randrange(args.hotzone + 1)
    if i == args.hotzone:
      i = randrange(args.hotzone)
      xs[i] = 1 + randrange(args.memory)
    yield xs[i]

def generate():
  if args.algo == 'random':
    M = args.memory // args.threads
    assert M > 0
    gs = [random_gen(i * M + 1, i * M + M + 1) for i in range(args.threads)]
  else:
    gs = [local_gen() for _ in range(args.threads)]
  for _ in range(args.length):
    yield next(choice(gs))

def main():
  global args
  args = argparser.parse_args()
  sys.stderr.write('{}\n'.format(args))
  seed(args.seed)
  sys.stdout.write('{}\n{}'.format(args.cache, args.length))
  for x in generate():
    sys.stdout.write(' {}'.format(x))
  sys.stdout.write('\n')

if __name__ == '__main__':
  main()
