#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/mman.h>

struct arena {
  int * memory;
  int size;
};


struct arena initialize(int size) {
  assert (size >= 5);
  struct arena arena = {
    .memory =
      mmap(0,
          size * sizeof(int),
          PROT_READ|PROT_WRITE,
          MAP_PRIVATE|MAP_ANONYMOUS,
          -1, 0),
    .size = size,
  };
  if (arena.memory == MAP_FAILED) {
    perror("can't allocate arena");
    exit(EXIT_FAILURE);
  }
  arena.memory[0] = 2;
  arena.memory[1] = 3;
  arena.memory[2] = 1;
  arena.memory[3] = size - 3;
  arena.memory[4] = size;
  return arena;
}

void cleanup(struct arena arena) {
  munmap(arena.memory, arena.size * sizeof(int));
}

int * alloc(struct arena arena, int n) {
  int i, j;
  assert (n >= 1);
  i = 0;
  while (1) {
    j = arena.memory[i + 1];
    printf("trying to allocate %d at %d\n", n, j);
    if (j >= arena.size) {
      fprintf(stderr, "not enough memory in arena\n");
      exit(EXIT_FAILURE);
    }
    if (arena.memory[j] >= n + 3) break;
    i = j;
  }
  printf("allocate at %d\n", j);
  arena.memory[j + n + 1] = arena.memory[j] - n - 1;
  arena.memory[j + n + 2] = arena.memory[j + 1];
  arena.memory[i + 1] = j + n + 1;
  arena.memory[j] = n + 1;
  return arena.memory + j + 1;
}

void del(struct arena arena, int * x) {
  int i, j, k;
  j = (x - arena.memory) - 1;
  printf("deallocate at %d\n", j);
  i = 0;
  while (1) {
    k = arena.memory[i + 1];
    if (j < k) break;
    i = k;
  }
  arena.memory[j + 1] = k;
  arena.memory[i + 1] = j;
  if (i + arena.memory[i] == j) {
    printf("glue %d %d\n", i, j);
    arena.memory[i] += arena.memory[j];
    arena.memory[i + 1] = k;
    j = i;
  }
  printf("j %d sz %d k %d\n", j, arena.memory[j], k);
  if (k < arena.size && j + arena.memory[j] == k) {
    printf("glue %d %d\n", j, k);
    arena.memory[j] += arena.memory[k];
    arena.memory[j + 1] = arena.memory[k + 1];
  }
}

int main() {
  struct arena arena = initialize(15);
  int * x = alloc(arena, 2);
  int * y = alloc(arena, 2);
  del(arena, x);
  x = alloc(arena, 3);
  del(arena, y);
  del(arena, x);
  printf("ARENA\n");
  for (int i =0; i<15; ++i) { printf("%d %d\n", i, arena.memory[i]); }
  cleanup(arena);

  return 0;
}
