#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int kmp_length;
char * kmp_needle;
int * kmp_failure;

void kmp_precompute(char * needle) {
  assert (kmp_needle == NULL);
  assert (kmp_failure == NULL);
  kmp_length = strlen(needle);
  kmp_needle = malloc(kmp_length + 1);
  if (kmp_needle == NULL) {
    perror("malloc");
    exit (EXIT_FAILURE);
  }
  strcpy(kmp_needle, needle);
  kmp_failure = malloc(kmp_length * sizeof(int));
  if (kmp_failure == NULL) {
    perror("malloc");
    exit (EXIT_FAILURE);
  }
  kmp_failure[0] = -1;
  for (int i = 1; i < kmp_length; ++i) {
    int j = kmp_failure[i - 1];
    while (j >= 0 && kmp_needle[j] != kmp_needle[i - 1]) j = kmp_failure[j];
    kmp_failure[i] = j + 1;
  }
}

void kmp_free() {
  free(kmp_needle);
  free(kmp_failure);
  kmp_needle = NULL;
  kmp_failure = NULL;
}

int kmp_find(int (*get)()) {
  assert (kmp_needle != NULL);
  assert (kmp_failure != NULL);
  int found = 0;
  int c;
  while ((c = get()) != EOF) {
    while (found >= 0 && c != kmp_needle[found]) found = kmp_failure[found];
    if (++found == kmp_length) return 1;
  }
  return 0;
}

