import java.util.*;

// Holds information about a thread. The name is {@code Task} to avoid any
// conflicts with {@code java.lang.Thread}.
class Task {
  final int startTime;
  final String name;
  final int duration;

  double virtualTime;
  int toGo;

  Task(final int startTime, final String name, final int duration) {
    this.startTime = startTime;
    this.name = name;
    this.duration = duration;
    virtualTime = startTime;
    toGo = duration;
  }

  public static final Comparator<Task> CompareByVirtualTime = new Comparator<Task>() {
    public int compare(Task a, Task b) {
      int c;
      if ((c = Double.compare(a.virtualTime, b.virtualTime)) != 0) return c;
      return a.name.compareTo(b.name);
    }
  };
}


public class CompletelyFairScheduler {
  public static void main(String[] args) {
    // parse input
    Scanner scan = new Scanner(System.in);
    int n = scan.nextInt();
    Deque<Task> input = new ArrayDeque<>();
    while (scan.hasNext()) {
      final int startTime = scan.nextInt();
      final String name = scan.next();
      final int duration = scan.nextInt();
      input.addLast(new Task(startTime, name, duration));
    }

    int time = 0; // initialize with simulation start time

    // Check whether the input is valid.
    { int lastTime = time;
      for (Task t : input) {
        if (t.startTime < lastTime) throw new IllegalArgumentException();
        lastTime = t.startTime;
      }
    }

    // schedule
    PriorityQueue<Task> active = new PriorityQueue<>(2, Task.CompareByVirtualTime);
    Deque<Task> running = new ArrayDeque<>();
    while (!active.isEmpty() || !running.isEmpty() || !input.isEmpty()) {
      // XXX: Add back this optimization?
      if (false) if (active.isEmpty() && running.isEmpty()) {
        // small optimization: skip uninteresting time steps
        time = input.getFirst().startTime;
      }

      // Handle requests to create threads.
      while (true) {
        Task t = input.peekFirst();
        if (t == null || t.startTime > time) break;
        input.removeFirst();
        active.add(t);
      }

      // Schedule work (for one time step).
      for (int i = 0; i < n && !active.isEmpty(); ++i) {
        running.addLast(active.remove());
      }

      // Report what is running
      if (!running.isEmpty()) {
        System.out.printf("@%d", time);
        int core = 0;
        for (Task t : running) {
          System.out.printf(" %s:%d:%.2f", t.name, ++core, t.virtualTime);
        }
        System.out.printf("\n");
      }

      // Put running tasks back in the queue, unless they finished.
      int m = active.size() + running.size();
      double delta = Math.max(1.0, 1.0 * m / n);
      while (!running.isEmpty()) {
        Task t = running.removeFirst();
        t.virtualTime += delta;
        --t.toGo;
        if (t.toGo > 0) active.add(t);
      }

      ++time;
    } // while (...)
  } // main
} // class
